/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include "CoreMinimal.h"
#include "MixCastDefines.h"
#if UE_4_18_OR_LATER
#include "UObject/SoftObjectPath.h"
#else
#include "Misc/StringClassReference.h"
#endif

#include "MixCast_Shaders.h"

#include "PluginProjectSettings.generated.h"

#if UE_4_25_OR_LATER
#define UPROPCLASS FProperty
#else
#define UPROPCLASS UProperty
#endif

UENUM()
namespace EMixCastClipMode
{
	enum Type
	{
		PerPixel,
		Planar
	};
}

UENUM()
namespace EMixCastTranslucencyMode
{
	enum Type
	{
		None,
		ForceAdditive,
		FromAlpha
	};
}

UCLASS(config = Game, defaultconfig)
class MIXCAST_API UPluginProjectSettings : public UObject
{
	GENERATED_BODY()
public:
	UPluginProjectSettings(const FObjectInitializer& ObjectInitializer);

	//Customizes which Actor is spawned for each configured MixCast Camera
	UPROPERTY(config, noclear, EditDefaultsOnly, BlueprintReadWrite, Category = "Customization", meta = (MetaClass = "ExpCamera"))
		FStringClassReference CameraClass;
	//Customizes which Actor is spawned for each configured MixCast Projection
	UPROPERTY(config, noclear, EditDefaultsOnly, BlueprintReadWrite, Category = "Customization", meta = (MetaClass = "ExpVideoInput"))
		FStringClassReference VideoInputClass;
	//Customizes which Actor is spawned for each configured MixCast Viewfinder
	UPROPERTY(config, noclear, EditDefaultsOnly, BlueprintReadWrite, Category = "Customization", meta = (MetaClass = "ExpViewfinder"))
		FStringClassReference ViewfinderClass;

	//Determines how the foreground is separated from the rest of the scene (Obsolete above UE 4.21)
	UPROPERTY(EditAnywhere, config, Category = "Rendering")
		TEnumAsByte<EMixCastClipMode::Type> ClipMode = EMixCastClipMode::PerPixel;

	//How the Translucency of the scene should be determined
	UPROPERTY(EditAnywhere, config, Category = "Rendering")
		TEnumAsByte<EMixCastTranslucencyMode::Type> TranslucencyMode = EMixCastTranslucencyMode::FromAlpha;

	//Tells MixCast that this experience can render the scene with a skybox/background so the user can be composited with the virtual background
	UPROPERTY(EditAnywhere, config, Category = "Rendering")
		bool CanRenderOpaqueBg = true;
	//Indicates that this experience can render the scene without a skybox/background so it can be composited with a physical background
	UPROPERTY(EditAnywhere, config, Category = "Rendering")
		bool CanRenderTransparentBg = false;

	//UPROPERTY(EditAnywhere, config, Category = "Rendering", meta = (HideAlphaChannel))
		FLinearColor BackgroundColor = FLinearColor::Black;

	UPROPERTY(EditAnywhere, config, Category = "Build")
		bool bRequireCommandLineFlag;

	void ValidateCurrentSettings();

#if UE_EDITOR
	static void Register();
	static void Unregister();

	virtual bool CanEditChange(const UPROPCLASS* InProperty) const override;
#if !MIXCAST_OCCLUDE_SCREENSPACE
	virtual void PostEditChangeProperty(struct FPropertyChangedEvent& PropertyChangedEvent) override;

	void EnforceGlobalClipPlaneSettingIfNeeded();
#endif
#endif
};
