#pragma once

#if PLATFORM_WINDOWS
#include "Windows/AllowWindowsPlatformTypes.h"
#endif
#include <Windows.h>
#include <stdio.h>
#include <conio.h>
#if PLATFORM_WINDOWS
#include "Windows/HideWindowsPlatformTypes.h"
#endif

#include "SharedQueueReader.hpp"

struct SharedQueueConsumerDesc
{
	int capacity;
};

template <class T>
class SharedQueueConsumer
{
private:
	HANDLE descFile = NULL;

	int capacity;
	SharedQueueReader<T>** elements;

	int advancedCount = 0;
public:
	int GetCurrentSlot()
	{
		return advancedCount % capacity;
	}

private:
	SharedQueueReader<T>* GetCurrentReader()
	{
		return elements[GetCurrentSlot()];
	}

public:
	SharedQueueConsumer(LPCWSTR chainId) {

		LPWSTR key = new TCHAR[256];

		//Metadata
		wsprintf(key, TEXT("MixCast-Chain(%s)-Desc"), chainId);
		descFile = OpenFileMapping(FILE_MAP_READ, false, key);
		if (descFile) 
		{
			SharedQueueConsumerDesc* sharedDesc = (SharedQueueConsumerDesc*)MapViewOfFile(descFile,   // handle to map object
				FILE_MAP_READ, // read/write permission
				0,
				0,
				sizeof(SharedQueueConsumerDesc));
			capacity = sharedDesc->capacity;
			UnmapViewOfFile(sharedDesc);

			elements = new SharedQueueReader<T>*[capacity];
			for (int i = 0; i < capacity; i++)
			{
				wsprintf(key, TEXT("%s[%d]"), chainId, i);
				elements[i] = new SharedQueueReader<T>(key);
			}
		}

		delete[] key;
	}
	~SharedQueueConsumer()
	{
		for (int i = 0; i < capacity; i++)
		{
			delete elements[i];
			elements[i] = NULL;
		}
		delete[] elements;

		CloseHandle(descFile);
	}

	int GetCapacity()
	{
		return capacity;
	}

	void EnqueueFenceUpdates()
	{
		for (int i = 0; i < capacity; i++)
			elements[i]->EnqueueFenceUpdate();
	}

	bool WaitUntilNextFilled(int timeout = 0)
	{
		return GetCurrentReader()->WaitUntilFilled(timeout);
	}
	void Read(T& dst)
	{
		GetCurrentReader()->Read(dst);
	}

	void MarkEmptied()
	{
		GetCurrentReader()->MarkEmptied();
		advancedCount++;
	}
	void EnqueueFenceInsert()
	{
		GetCurrentReader()->EnqueueFenceInsert();
		advancedCount++;
	}
};

